﻿using ProjectBulldozer.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectBulldozer.Drawnings;

/// <summary>
/// Класс отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningDozer
{
    ///<summary>
    /// Класс-сущность
    /// </summary>
    public EntityDozer? EntityDozer { get; protected set; }

    ///<summary>
    ///Ширина окна
    /// </summary>
    private int? _pictureWidth;

    ///<summary>
    ///Высота окна
    /// </summary>
    private int? _pictureHeight;

    ///<summary>
    /// Левая координата прорисовки бульдозера
    /// </summary>
    protected int? _startPosX;

    ///<summary>
    /// Верхнаяя координата прорисовки бульдозера
    /// </summary>
    protected int? _startPosY;
    private EntityDozer dozer;

    ///<summary>
    ///Ширина прорисовки бульдозера
    /// </summary>
    private readonly int _drawningBulldozerWidth = 150;

    ///<summary>
    ///Высота прорисовки бульдозера
    /// </summary>
    private readonly int _drawningBulldozerHeight = 90;

    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningBulldozerWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningBulldozerHeight;


    /// <summary>
    /// Пустой конструктор.
    /// </summary>
    private DrawningDozer() 
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет(цвет колёс)</param>
    public DrawningDozer(int speed, double weight, Color bodyColor) : this()
    {
        EntityDozer = new EntityDozer(speed, weight, bodyColor);
    }

    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningBulldozerWidth">Ширина прорисовки бульдозера</param>
    /// <param name="drawningBulldozerHeigh">Высота прорисовки бульдозера</param>
    protected DrawningDozer (int drawningBulldozerWidth, int drawningBulldozerHeigh) :this() 
    {
        _drawningBulldozerWidth = drawningBulldozerWidth;
        _pictureHeight = drawningBulldozerHeigh;
    }

    /// <summary>
    /// Другой конструктор.
    /// </summary>
    /// <param name="dozer"></param>
    public DrawningDozer(EntityDozer? entityDozer)
    {
        if (entityDozer == null) return;
        EntityDozer = entityDozer;
    }

    ///<summary>
    ///Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        //проверка, что объект "влезает" в размеры поля. Если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена.
        if (width < _drawningBulldozerWidth || height < _drawningBulldozerHeight)
        {
            return false;
        }
        else
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX != null || _startPosY != null)
            {
                if (_startPosX < 0)
                {
                    _startPosX = 0;
                }
                if (_startPosX + _drawningBulldozerWidth >= _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawningBulldozerWidth;
                }
                if (_startPosY < 0)
                {
                    _startPosY = 0;
                }
                if (_startPosY + _drawningBulldozerHeight >= _pictureHeight)
                {
                    _startPosY = _pictureHeight - _drawningBulldozerHeight;
                }
            }
        }

        return true;
    }

    ///<summary>
    ///Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        //если при установке объекта в эти координаты, он будет выходить за границы формы, то надо изменить координаты,
        //чтобы он оставался в этих границах
        if (x < 0)
        {
            _startPosX = 0;
        }
        else if (x + _drawningBulldozerWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _drawningBulldozerWidth;
        }
        else
        {
            _startPosX = x;
        }

        if (y < 0)
        {
            _startPosY = 0;
        }
        else if (y + _drawningBulldozerHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _drawningBulldozerHeight;
        }
        else
        {
            _startPosY = y;
        }
    }

    ///<summary>
    ///Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направлениие</param>
    /// <returns>true - перемещениие выполнено, false - перемещение невозможно</returns>
    public virtual bool MoveTransport(DirectionType direction)
    {
        if (EntityDozer == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            // влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityDozer.Step > 0)
                {
                    _startPosX -= (int)EntityDozer.Step;
                }
                return true;
            // вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityDozer.Step > 0)
                {
                    _startPosY -= (int)EntityDozer.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityDozer.Step + _drawningBulldozerWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityDozer.Step;
                }
                return true;
            // вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityDozer.Step + _drawningBulldozerHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityDozer.Step;
                }
                return true;

            default:
                return false;
        }
    }
    ///<summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityDozer == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush bodyBrush = new SolidBrush(EntityDozer.BodyColor);
        Brush wheelstringBuilderrush = new SolidBrush(Color.FromArgb(60, 60, 60));
        //BULDOZER
        //body
        g.FillRectangle(bodyBrush, _startPosX.Value + 10, _startPosY.Value + 15, _drawningBulldozerWidth - 20, _drawningBulldozerHeight - 30);
        g.DrawRectangle(pen, _startPosX.Value + 10, _startPosY.Value + 15, _drawningBulldozerWidth - 20, _drawningBulldozerHeight - 30);
        //wheels
        g.FillRectangle(wheelstringBuilderrush, _startPosX.Value, _startPosY.Value + _drawningBulldozerHeight - 15, 50, 15);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + _drawningBulldozerHeight - 15, 50, 15);
        g.FillRectangle(wheelstringBuilderrush, _startPosX.Value + _drawningBulldozerWidth - 50, _startPosY.Value + _drawningBulldozerHeight - 15, 50, 15);
        g.DrawRectangle(pen, _startPosX.Value + _drawningBulldozerWidth - 50, _startPosY.Value + _drawningBulldozerHeight - 15, 50, 15);
        g.FillRectangle(wheelstringBuilderrush, _startPosX.Value, _startPosY.Value, 50, 15);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value, 50, 15);
        g.FillRectangle(wheelstringBuilderrush, _startPosX.Value + _drawningBulldozerWidth - 50, _startPosY.Value, 50, 15);
        g.DrawRectangle(pen, _startPosX.Value + _drawningBulldozerWidth - 50, _startPosY.Value, 50, 15);
        //strange rectangles
        g.DrawRectangle(pen, _startPosX.Value + 10, _startPosY.Value + 25, 3, _drawningBulldozerHeight - 50);
        g.DrawRectangle(pen, _startPosX.Value + 13, _startPosY.Value + 25, 20, 4);
        g.DrawRectangle(pen, _startPosX.Value + 13, _startPosY.Value + _drawningBulldozerHeight - 29, 20, 4);
        g.DrawRectangle(pen, _startPosX.Value + 13, _startPosY.Value + 29, 17, _drawningBulldozerHeight - 58);
        //strange circles
        g.DrawEllipse(pen, _startPosX.Value + 45, _startPosY.Value + 42, 6, 6);
        g.DrawEllipse(pen, _startPosX.Value + 45, _startPosY.Value + 30, 6, 6);
        g.DrawEllipse(pen, _startPosX.Value + 45, _startPosY.Value + 54, 6, 6);
        //window
        Brush windowBrush = new SolidBrush(Color.FromArgb(170, 170, 215));
        g.FillRectangle(windowBrush, _startPosX.Value + 55, _startPosY.Value + 20, _drawningBulldozerWidth - 90, _drawningBulldozerHeight - 40);
        g.DrawRectangle(pen, _startPosX.Value + 55, _startPosY.Value + 20, _drawningBulldozerWidth - 90, _drawningBulldozerHeight - 40);
    }
}